/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "client.h"
#include "assert/assert.h"
#include "console/console.h"

void* clientRemoteThreadFunc(void* arg) {
  client_t *client = (client_t*)arg;
  assertNotNull(client, "Client is NULL");

  client->state = CLIENT_STATE_CONNECTED;

  // Send some data
  write(client->remote.clientSockDesc, "Hello, World!", 13);
  
  printf("Thread func\n");

  return NULL;
}

void clientRemoteInit(
  client_t *client,
  const clientremoteinit_t init
) {
  client->remote.clientSockDesc = init.clientSockDesc;
  consolePrint("Accepted a new connection.");

  pthread_create(&client->remote.thread, NULL, clientRemoteThreadFunc, client);
}

void clientRemoteSend(client_t *client, const packet_t packet) {
  printf("send pack\n");
}

void clientRemoteDispose(client_t *client) {
  pthread_join(client->remote.thread, NULL);

  close(client->remote.clientSockDesc);
  consolePrint("Client disconnected.");
}