/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "console.h"
#include "assert/assert.h"

console_t CONSOLE;

void consoleInit() {
  memset(&CONSOLE, 0, sizeof(console_t));
}

const consolecmd_t * consoleCommand(const char_t *cmd) {
  assertTrue(
    CONSOLE.commandCount < CONSOLE_COMMAND_COUNT_MAX,
    "Command buffer full."
  );
  assertStrLen(cmd, CONSOLE_CMD_LENGTH_MAX, "Command too long.");

  consolecmd_t *command = &CONSOLE.commands[CONSOLE.commandCount++];
  strcpy(command->cmd, cmd);
  return command;
}

void consolePrint(const char_t *fmt, ...) {
  // Do we need to shift the log buffer?
  if (CONSOLE.logCount == CONSOLE_LOG_COUNT_MAX) {
    memcpy(
      CONSOLE.log[0],
      CONSOLE.log[1],
      sizeof(char_t) *
      CONSOLE_LOG_MESSAGE_LENGTH_MAX *
      (CONSOLE_LOG_COUNT_MAX - 1)
    );
  }

  // Print the message
  va_list args;
  va_start(args, fmt);

  // buffer but don't let it overflow the string length.
  int ret = vsnprintf(
    CONSOLE.log[CONSOLE.logCount++],
    CONSOLE_LOG_MESSAGE_LENGTH_MAX,
    fmt,
    args
  );

  assertTrue(ret >= 0, "Log message error.");
  assertTrue(ret < CONSOLE_LOG_MESSAGE_LENGTH_MAX, "Log message overflow.");
  
  printf("%s\n", CONSOLE.log[CONSOLE.logCount - 1]);

  va_end(args);
}