/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "consolecmd.h"

#define CONSOLE_COMMAND_COUNT_MAX 64
#define CONSOLE_LOG_MESSAGE_LENGTH_MAX 1024
#define CONSOLE_LOG_COUNT_MAX 256

typedef struct {
  consolecmd_t commands[CONSOLE_COMMAND_COUNT_MAX];
  uint8_t commandCount;

  char_t log[CONSOLE_LOG_COUNT_MAX][CONSOLE_LOG_MESSAGE_LENGTH_MAX + 1];
  uint8_t logCount;
} console_t;

extern console_t CONSOLE;

/**
 * Initializes the console.
 */
void consoleInit();

/**
 * Run a command against the console.
 * 
 * @param cmd The command string to run.
 */
const consolecmd_t * consoleCommand(const char_t *cmd);

/**
 * Prints a message to the console.
 * 
 * @param fmt The format string.
 * @param ... The format arguments.
 */
void consolePrint(const char_t *fmt, ...);