/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"

typedef struct {
  mat4 projection;
  mat4 view;
} camera_t;

/**
 * Initializes the camera.
 * 
 * @param camera The camera to initialize.
 */
void cameraInit(camera_t *camera);

/**
 * Changes a camera to look at a specific point.
 * 
 * @param camera The camera to modify.
 * @param eye The position of the camera.
 * @param center The point to look at.
 * @param up The up vector.
 */
void cameraLookAt(
  camera_t *camera,
  const vec3 eye,
  const vec3 center,
  const vec3 up
);

/**
 * Changes the camera's perspective.
 * 
 * @param camera The camera to modify.
 * @param fovy The field of view in the y direction.
 * @param aspect The aspect ratio.
 * @param zNear The near plane.
 * @param zFar The far plane.
 */
void cameraPerspective(
  camera_t *camera,
  const float fovy,
  const float aspect,
  const float zNear,
  const float zFar
);

/**
 * Changes the camera's orthographic perspective.
 * 
 * @param camera The camera to modify.
 * @param left The left plane.
 * @param right The right plane.
 * @param bottom The bottom plane.
 * @param top The top plane.
 * @param near The near plane.
 * @param far The far plane.
 */
void cameraOrtho(
  camera_t *camera,
  const float left,
  const float right,
  const float bottom,
  const float top,
  const float near,
  const float far
);