/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "input.h"

inputstate_t INPUT;

void inputInit() {
  memset(&INPUT, 0, sizeof(inputstate_t));
}

void inputUpdate() {
  memcpy(INPUT.previous, INPUT.current, sizeof(inputvalue_t) * INPUT_BIND_COUNT);
  
  for(int i = 0; i < INPUT_BIND_COUNT; i++) {
    INPUT.current[i] = inputStateGet(i);
  }
}

inputvalue_t inputGet(inputbind_t bind) {
  return INPUT.current[bind];
}

inputvalue_t inputGetPrevious(inputbind_t bind) {
  return INPUT.previous[bind];
}

bool_t inputWasPressed(inputbind_t bind) {
  return inputGet(bind) != 0.0f && inputGetPrevious(bind) == 0.0f;
}

bool_t inputWasReleased(inputbind_t bind) {
  return inputGet(bind) == 0.0f && inputGetPrevious(bind) != 0.0f;
}

bool_t inputIsDown(inputbind_t bind) {
  return inputGet(bind) != 0.0f;
}

bool_t inputIsUp(inputbind_t bind) {
  return inputGet(bind) == 0.0f;
}