/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"

typedef float_t inputvalue_t;

typedef enum {
  INPUT_UP,
  INPUT_DOWN,
  INPUT_LEFT,
  INPUT_RIGHT,
  INPUT_ACCEPT,
  INPUT_BACK,
  INPUT_QUIT,
} inputbind_t;

#define INPUT_BIND_COUNT INPUT_QUIT + 1

typedef struct {
  inputvalue_t current[INPUT_BIND_COUNT];
  inputvalue_t previous[INPUT_BIND_COUNT];
} inputstate_t;

extern inputstate_t INPUT;

/**
 * Initializes the input system.
 */
void inputInit();

/**
 * Updates the input system.
 */
void inputUpdate();

/**
 * Calls on the platform to get the current state of a given input bind.
 * 
 * @param input Input to check.
 * @return Current state of the input.
 */
inputvalue_t inputStateGet(const inputbind_t input);

/**
 * Returns the current value of the input bind.
 * 
 * @param input Input to get.
 * @return Current value of the input.
 */
inputvalue_t inputGet(const inputbind_t input);

/**
 * Returns the previous value of the input bind.
 * 
 * @param input Input to get.
 * @return Previous value of the input.
 */
inputvalue_t inputGetPrevious(const inputbind_t input);

/**
 * Returns whether the input was pressed this frame.
 * 
 * @param input Input to check.
 * @return Whether the input was pressed this frame.
 */
bool_t inputWasPressed(inputbind_t input);

/**
 * Returns whether the input was released this frame.
 * 
 * @param input Input to check.
 * @return Whether the input was released this frame.
 */
bool_t inputWasReleased(inputbind_t input);

/**
 * Returns whether the input is currently down.
 * 
 * @param input Input to check.
 * @return Whether the input is currently down.
 */
bool_t inputIsDown(inputbind_t input);

/**
 * Returns whether the input is currently up.
 * 
 * @param input Input to check.
 * @return Whether the input is currently up.
 */
bool_t inputIsUp(inputbind_t input);