/**
 * Copyright (c) 2023 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"

/**
 * Returns the minimum of two values.
 * @param a First value.
 * @param b Second value.
 * @return The minimum of the two values.
 */
#define mathMin(a, b) ((a) < (b) ? (a) : (b))

/**
 * Returns the maximum of two values.
 * @param a First value.
 * @param b Second value.
 * @return The maximum of the two values.
 */
#define mathMax(a, b) ((a) > (b) ? (a) : (b))

/**
 * Clamps a value between a min and max.
 * 
 * @param value Value to clamp.
 * @param min Minimum value to clamp value to.
 * @param max Maximum value to clamp value to.
 * @return The clamped value.
 */
int32_t mathClampi32(
  const int32_t value,
  const int32_t min,
  const int32_t max
);

/**
 * Returns the modulo of a value. For 32 bit integers.
 * 
 * @param value Value to modulo.
 * @param modulo Modulo value.
 * @return The modulo of the value.
 */
int32_t mathModi32(int32_t value, int32_t modulo);