/**
 * Copyright (c) 2023 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "assert/assertgl.h"
#include "assert/assert.h"

void assertNotGLErrorCheck(const char_t *file, const int32_t line) {
  assertNotNull(file, "File is an invlaid string");
  assertTrue(line > 0, "assertNotGLErrorCheck: line is invalid");

  char_t *buffer;
  GLenum errorCode;
  int32_t errorCount = 0;

  while((errorCode = glGetError()) != GL_NO_ERROR) {
    if(errorCount == 0) {
      buffer = malloc(sizeof(char_t) * 4096);
      sprintf(buffer, "assertNotGLErrorCheck: %s:%d\n", file, line);
    }

    errorCount++;
    
    switch (errorCode) {
      case GL_INVALID_ENUM:
        sprintf(buffer, "%s\nINVALID_ENUM", buffer);
        break;

      case GL_INVALID_VALUE:
        sprintf(buffer, "%s\nINVALID_ENUM", buffer);
        break;
      
      case GL_INVALID_OPERATION:
        sprintf(buffer, "%s\nINVALID_OPERATION", buffer);
        break;

      // case GL_INVALID_FRAMEBUFFER_OPERATION:
      //   sprintf(buffer, "%s\nINVALID_FRAMEBUFFER_OPERATION", buffer);
      //   break;
      
      case GL_OUT_OF_MEMORY:
        sprintf(buffer, "%s\nOUT_OF_MEMORY", buffer);
        break;
      
      case GL_STACK_UNDERFLOW:
        sprintf(buffer, "%s\nSTACK_UNDERFLOW", buffer);
        break;
      
      case GL_STACK_OVERFLOW:
        sprintf(buffer, "%s\nSTACK_OVERFLOW", buffer);
        break;

      default:
        sprintf(buffer, "%s\nUNKNOWN_ERROR", buffer);
        break;
    }

    sprintf(buffer, "%s (%i)\n", buffer, errorCode);
  }

  if(errorCount > 0) {
    assertUnreachable(buffer);
    free(buffer);
  }
}