/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "window.h"

GLFWwindow* window;
int32_t WINDOW_WIDTH;
int32_t WINDOW_HEIGHT;

int32_t windowInit() {
  // Initialize GLFW
  if(!glfwInit()) {
    const char* description;
    int code = glfwGetError(&description);
    if (description) {
      printf("GLFW Error %d: %s\n", code, description);
    } else {
      printf("GLFW Error: Unknown error\n");
    }
    
    return -1;
  }

  // Create a windowed mode window and its OpenGL contex
  glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
  glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
  glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
  glfwWindowHint(GLFW_RESIZABLE, GLFW_TRUE);
  glfwWindowHint(GLFW_MAXIMIZED, GLFW_FALSE);

  window = glfwCreateWindow(
    WINDOW_WIDTH_DEFAULT, WINDOW_HEIGHT_DEFAULT,
    "Dusk",
    NULL, NULL
  );
  if(!window) {
    glfwTerminate();
    return -2;
  }

  // Make the window's context current
  glfwMakeContextCurrent(window);

  // Get initial framebuffer size
  glfwGetFramebufferSize(window, &WINDOW_WIDTH, &WINDOW_HEIGHT);

  // Load OpenGL functions using glad or another loader here if necessary
  if(!gladLoadGLLoader((GLADloadproc)glfwGetProcAddress)) return -3;
  
  return 0;
}

bool_t windowShouldClose() {
  return glfwWindowShouldClose(window);
}

void windowUpdate() {
  glfwSwapBuffers(window);
  glfwPollEvents();

  glfwGetFramebufferSize(window, &WINDOW_WIDTH, &WINDOW_HEIGHT);
}

void windowDispose() {
  glfwDestroyWindow(window);
  glfwTerminate();
}