/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "input.h"
#include "display/window.h"

typedef struct {
  uint16_t glfw;
  inputbind_t bind;
} glfwkeybindmap_t;

glfwkeybindmap_t GLFW_KEY_BIND_MAP[] = {
  {GLFW_KEY_W, INPUT_UP},
  {GLFW_KEY_UP, INPUT_UP},
  
  {GLFW_KEY_S, INPUT_DOWN},
  {GLFW_KEY_DOWN, INPUT_DOWN},
  
  {GLFW_KEY_A, INPUT_LEFT},
  {GLFW_KEY_LEFT, INPUT_LEFT},
  
  {GLFW_KEY_D, INPUT_RIGHT},
  {GLFW_KEY_RIGHT, INPUT_RIGHT},
  
  {GLFW_KEY_ENTER, INPUT_ACCEPT},
  {GLFW_KEY_E, INPUT_ACCEPT},
  {GLFW_KEY_SPACE, INPUT_ACCEPT},

  {GLFW_KEY_ESCAPE, INPUT_BACK},
  {GLFW_KEY_BACKSPACE, INPUT_BACK},
  {GLFW_KEY_Q, INPUT_BACK},
  
  {0, 0}
};

inputvalue_t inputStateGet(const inputbind_t input) {

  // Handle keybinds
  glfwkeybindmap_t *map = GLFW_KEY_BIND_MAP;
  do {
    if(map->bind == input && glfwGetKey(window, map->glfw) == GLFW_PRESS) {
      return 1;
    }
    map++;
  } while(map->glfw != 0);

  return 0;
}